/*
 * Jalview - A Sequence Alignment Editor and Viewer
 * Copyright (C) 2005 AM Waterhouse, J Procter, G Barton, M Clamp, S Searle
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package jalview.gui;

import jalview.jbgui.*;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;

import javax.swing.*;


/**
 * Base class for web service client thread and gui
 *
 * @author $author$
 * @version $Revision$
 */
public class WebserviceInfo extends GWebserviceInfo
{
    /** Job is Queued */
    public static final int STATE_QUEUING = 0;

    /** Job is Running */
    public static final int STATE_RUNNING = 1;

    /** Job has finished with no errors */
    public static final int STATE_STOPPED_OK = 2;

    /** Job has been cancelled with no errors */
    public static final int STATE_CANCELLED_OK = 3;

    /** job has stopped because of some error */
    public static final int STATE_STOPPED_ERROR = 4;

    /** job has failed because of some unavoidable service interruption */
    public static final int STATE_STOPPED_SERVERERROR = 5;
    int currentStatus = STATE_QUEUING;
    Image image;
    int angle = 0;
    String title = "";
    jalview.ws.WSClientI thisService;
    boolean serviceIsCancellable;
    JInternalFrame frame;

    /**
     * Creates a new WebserviceInfo object.
     *
     * @param title short name and job type
     * @param info reference or other human readable description
     */
    public WebserviceInfo(String title, String info)
    {
        init(title, info, 520, 500);
    }

    /**
     * Creates a new WebserviceInfo object.
     *
     * @param title DOCUMENT ME!
     * @param info DOCUMENT ME!
     * @param width DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    public WebserviceInfo(String title, String info, int width, int height)
    {
        init(title, info, width, height);
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public jalview.ws.WSClientI getthisService()
    {
        return thisService;
    }

    /**
     * DOCUMENT ME!
     *
     * @param newservice DOCUMENT ME!
     */
    public void setthisService(jalview.ws.WSClientI newservice)
    {
        thisService = newservice;
        serviceIsCancellable = newservice.isCancellable();
    }

    /**
     * DOCUMENT ME!
     *
     * @param title DOCUMENT ME!
     * @param info DOCUMENT ME!
     * @param width DOCUMENT ME!
     * @param height DOCUMENT ME!
     */
    void init(String title, String info, int width, int height)
    {
        frame = new JInternalFrame();
        frame.setContentPane(this);
        Desktop.addInternalFrame(frame, title, width, height);
        frame.setClosable(false);

        this.title = title;
        setInfoText(info);

        java.net.URL url = getClass().getResource("/images/logo.gif");
        image = java.awt.Toolkit.getDefaultToolkit().createImage(url);

        MediaTracker mt = new MediaTracker(this);
        mt.addImage(image, 0);

        try
        {
            mt.waitForID(0);
        }
        catch (Exception ex)
        {
        }

        AnimatedPanel ap = new AnimatedPanel();
        titlePanel.add(ap, BorderLayout.CENTER);

        Thread thread = new Thread(ap);
        thread.start();
    }

    /**
     * DOCUMENT ME!
     *
     * @param status integer status from state constants
     */
    public void setStatus(int status)
    {
        currentStatus = status;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getInfoText()
    {
        return infoText.getText();
    }

    /**
     * DOCUMENT ME!
     *
     * @param text DOCUMENT ME!
     */
    public void setInfoText(String text)
    {
        infoText.setText(text);
    }

    /**
     * DOCUMENT ME!
     *
     * @param text DOCUMENT ME!
     */
    public void appendInfoText(String text)
    {
        infoText.append(text);
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public String getProgressText()
    {
        return progressText.getText();
    }

    /**
     * DOCUMENT ME!
     *
     * @param text DOCUMENT ME!
     */
    public void setProgressText(String text)
    {
        progressText.setText(text);
    }

    /**
     * DOCUMENT ME!
     *
     * @param text DOCUMENT ME!
     */
    public void appendProgressText(String text)
    {
        progressText.append(text);
    }

    /**
     * Gui action for cancelling the current job, if possible.
     *
     * @param e DOCUMENT ME!
     */
    protected void cancel_actionPerformed(ActionEvent e)
    {
        if (!serviceIsCancellable)
        {
            JOptionPane.showInternalMessageDialog(Desktop.desktop,
                "This job cannot be cancelled.\nJust close the window.", "Cancel job",
                JOptionPane.WARNING_MESSAGE);
        }
        else
        {
            frame.setClosable(true);
            thisService.cancelJob();
        }
    }

    public void setResultsReady()
    {
      frame.setClosable(true);
      buttonPanel.remove(cancel);
      buttonPanel.add(showResultsNewFrame);
      buttonPanel.add(mergeResults);
      buttonPanel.setLayout(new GridLayout(2,1,5,5));
      buttonPanel.validate();
      validate();
    }


    class AnimatedPanel extends JPanel implements Runnable
    {
        long startTime = 0;
        BufferedImage offscreen;

        public void run()
        {
            startTime = System.currentTimeMillis();
            Graphics2D g = null;

            while (currentStatus < STATE_STOPPED_OK)
            {
                try
                {
                    Thread.sleep(50);

                    int units = (int) ( (System.currentTimeMillis() - startTime) /
                                       10f);
                    angle += units;
                    angle %= 360;
                    startTime = System.currentTimeMillis();

                    if (offscreen == null || offscreen.getWidth(this) != getWidth()
                        || offscreen.getHeight(this) != getHeight())
                    {
                      offscreen = new BufferedImage(getWidth(), getHeight(),
                                                    BufferedImage.TYPE_INT_ARGB);
                      g = (Graphics2D) offscreen.getGraphics();
                    }

                    g.setColor(Color.white);
                    g.fillRect(0, 0, getWidth(), getHeight());

                    g.setFont(new Font("Arial", Font.BOLD, 12));
                    g.setColor(Color.black);

                    switch (currentStatus)
                    {
                      case STATE_QUEUING:
                        g.drawString(title.concat(" - queuing"), 60, 30);

                        break;

                      case STATE_RUNNING:
                        g.drawString(title.concat(" - running"), 60, 30);

                        break;

                      case STATE_STOPPED_OK:
                        g.drawString(title.concat(" - complete"), 60, 30);

                        break;

                      case STATE_CANCELLED_OK:
                        g.drawString(title.concat(" - job cancelled!"), 60, 30);

                        break;

                      case STATE_STOPPED_ERROR:
                        g.drawString(title.concat(" - job error!"), 60, 30);

                        break;

                      case STATE_STOPPED_SERVERERROR:
                        g.drawString(title.concat(" - Server Error! (try later)"),
                                     60,
                                     30);

                        break;
                    }


                    if (currentStatus >= STATE_STOPPED_OK)
                      angle = 0;

                    if (image != null)
                    {
                      g.rotate(Math.toRadians(angle), 28, 28);
                      g.drawImage(image, 10, 10, this);
                      g.rotate( -Math.toRadians(angle), 28, 28);
                    }


                    repaint();
                }
                catch (Exception ex)
                {
                }
            }

            cancel.setEnabled(false);
        }

        public void paintComponent(Graphics g1)
        {
            g1.drawImage(offscreen, 0,0,this);
        }
    }
}
